@@ -1,6 +1,6 @@
 #  MIT License
 #
-#  Copyright (c) 2017-2020 TileDB Inc.
+#  Copyright (c) 2017-2021 TileDB Inc.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -96,7 +96,7 @@ setMethod("schema", "tiledb_sparse", function(object, ...) {
   return(tiledb_array_schema.from_ptr(schema_xptr))
 })
 
-sparse_attribute_buffers <- function(array, sch, dom, sub, selected) {
+sparse_attribute_buffers <- function(array, sch, dom, sub, selected, ncells=1000) {
   stopifnot(is(sch, "tiledb_array_schema"))
   stopifnot(is(dom, "tiledb_domain"))
   #domaintype <- libtiledb_domain_get_type(dom@ptr)
@@ -104,9 +104,6 @@ sparse_attribute_buffers <- function(array, sch, dom, sub, selected) {
                        libtiledb_dim_get_datatype)
   attributes <- list()
   # first alloc coordinate buffer
-  #print(domaintype)
-  ncells <- libtiledb_array_max_buffer_elements_with_type(array@ptr, sub,
-                                                          libtiledb_coords(), domaintype[1])
   attributes[["coords"]] <- libtiledb_query_buffer_alloc_ptr(array@ptr, domaintype[1], ncells)
   attrs <- tiledb::attrs(sch)
   if (length(selected) == 0) {          # no selection given -> use all
@@ -121,10 +118,8 @@ sparse_attribute_buffers <- function(array, sch, dom, sub, selected) {
     dtype <- tiledb::datatype(attr)
     type <- tiledb_datatype_R_type(dtype)
     datatype <- libtiledb_attribute_get_type(attr@ptr)
-    #cat("dtype:", dtype, " type:", type, " datatype:", datatype, "\n", sep="")
-    ncells <- libtiledb_array_max_buffer_elements_with_type(array@ptr, sub, aname, domaintype[1])
     if (dtype %in% c("CHAR")) {  # TODO: add other char and date types
-      buff <- libtiledb_query_buffer_var_char_alloc(array@ptr, sub, aname)
+      buff <- libtiledb_query_buffer_var_char_alloc_direct(ncells, ncells*8, FALSE, sub[4]-sub[3]+1)
     } else if (datatype %in% c("DATETIME_DAY", "DATETIME_SEC", "DATETIME_MS",
                                "DATETIME_US", "DATETIME_NS")) {
       buff <- libtiledb_query_buffer_alloc_ptr(array@ptr, datatype, ncells)
