@@ -28,11 +28,12 @@
 #' based on refactored implementation utilising newer TileDB features.
 #'
 #' @slot ctx A TileDB context object
-#' @slot uri A character despription
-#' @slot is.sparse A logical value
+#' @slot uri A character despription with the array URI
+#' @slot is.sparse A logical value whether the array is sparse or not
 #' @slot as.data.frame A logical value
-#' @slot attrs A character vector
-#' @slot extended A logical value
+#' @slot attrs A character vector to select particular column \sQuote{attributes}
+#' @slot extended A logical value, defaults to \code{TRUE}, indicating whether index
+#' columns are returned as well.
 #' @slot selected_ranges An optional list with matrices where each matrix i
 #' describes the (min,max) pair of ranges for dimension i
 #' @slot query_layout An optional character value
@@ -53,6 +54,9 @@
 #' \sQuote{query_statistics} of the return object.
 #' @slot sil An optional and internal list object with schema information, used for
 #' parsing queries.
+#' @slot dumpbuffers An optional character variable with a directory name (relative to
+#' \code{/dev/shm}) for writing out results buffers (for internal use / testing)
+#' @slot buffers An optional list with full pathnames of shared memory buffers to read data from
 #' @slot ptr External pointer to the underlying implementation
 #' @exportClass tiledb_array
 setClass("tiledb_array",
@@ -75,6 +79,8 @@ setClass("tiledb_array",
                       return_as = "character",
                       query_statistics = "logical",
                       sil = "list",
+                      dumpbuffers = "character",
+                      buffers = "list",
                       ptr = "externalptr"))
 
 #' Constructs a tiledb_array object backed by a persisted tiledb array uri
@@ -88,7 +94,7 @@ setClass("tiledb_array",
 #' @param attrs optional character vector to select attributes, default is
 #' empty implying all are selected
 #' @param extended optional logical switch selecting wide \sQuote{data.frame}
-#' format, defaults to "TRUE"
+#' format, defaults to \code{TRUE}
 #' @param selected_ranges optional A list with matrices where each matrix i
 #' describes the (min,max) pair of ranges for dimension i
 #' @param query_layout optional A value for the TileDB query layout, defaults to
@@ -120,6 +126,9 @@ setClass("tiledb_array",
 #' \sQuote{query_statistics} of the return object.
 #' @param sil optional A list, by default empty to store schema information when query objects are
 #' parsed.
+#' @param dumpbuffers An optional character variable with a directory name (relative to
+#' \code{/dev/shm}) for writing out results buffers (for internal use / testing)
+#' @param buffers An optional list with full pathnames of shared memory buffers to read data from
 #' @param ctx optional tiledb_ctx
 #' @return tiledb_array object
 #' @export
@@ -142,6 +151,8 @@ tiledb_array <- function(uri,
                          return_as = get_return_as_preference(),
                          query_statistics = FALSE,
                          sil = list(),
+                         dumpbuffers = character(),
+                         buffers = list(),
                          ctx = tiledb_get_context()) {
   query_type = match.arg(query_type)
   if (!is(ctx, "tiledb_ctx"))
@@ -213,6 +224,8 @@ tiledb_array <- function(uri,
       return_as = return_as,
       query_statistics = query_statistics,
       sil = sil,
+      dumpbuffers = dumpbuffers,
+      buffers = buffers,
       ptr = array_xptr)
 }
 
@@ -498,10 +511,17 @@ setMethod("[", "tiledb_array",
     attrnullable <- attrnullable[ind]
   }
 
-  allnames <- c(dimnames, attrnames)
-  alltypes <- c(dimtypes, attrtypes)
-  allvarnum <- c(dimvarnum, attrvarnum)
-  allnullable <- c(dimnullable, attrnullable)
+  if (x@extended) {                     # if true return dimensions and attributes
+      allnames <- c(dimnames, attrnames)
+      alltypes <- c(dimtypes, attrtypes)
+      allvarnum <- c(dimvarnum, attrvarnum)
+      allnullable <- c(dimnullable, attrnullable)
+  } else {                              # otherwise only return attributes
+      allnames <- attrnames
+      alltypes <- attrtypes
+      allvarnum <- attrvarnum
+      allnullable <- attrnullable
+  }
 
   if (length(enckey) > 0) {
     if (length(tstamp) > 0) {
@@ -613,124 +633,142 @@ setMethod("[", "tiledb_array",
     }
   }
 
-  ## retrieve est_result_size
-  getEstimatedSize <- function(name, varnum, nullable, qryptr, datatype) {
-    if (is.na(varnum) && !nullable)
-      res <- libtiledb_query_get_est_result_size_var(qryptr, name)[1]
-    else if (is.na(varnum) && nullable)
-      res <- libtiledb_query_get_est_result_size_var_nullable(qryptr, name)[1]
-    else if (!is.na(varnum) && !nullable)
-      res <- libtiledb_query_get_est_result_size(qryptr, name)
-    else if (!is.na(varnum) && nullable)
-      res <- libtiledb_query_get_est_result_size_nullable(qryptr, name)[1]
-    if (rangeunset && tiledb::tiledb_version(TRUE) >= "2.2.0") {
-      sz <- switch(datatype,
-                   "UINT8"   = ,
-                   "INT8"    = 1,
-                   "UINT16"  = ,
-                   "INT16"   = 2,
-                   "INT32"   = ,
-                   "UINT32"  = ,
-                   "FLOAT32" = 4,
-                   "UINT64"  = ,
-                   "INT64"   = ,
-                   "FLOAT64" = ,
-                   "DATETIME_YEAR" = ,
-                   "DATETIME_MONTH" = ,
-                   "DATETIME_WEEK" = ,
-                   "DATETIME_DAY" = ,
-                   "DATETIME_HR" = ,
-                   "DATETIME_MIN" = ,
-                   "DATETIME_SEC" = ,
-                   "DATETIME_MS" = ,
-                   "DATETIME_US" = ,
-                   "DATETIME_NS" = ,
-                   "DATETIME_PS" = ,
-                   "DATETIME_FS" = ,
-                   "DATETIME_AS" = 8,
-                   1)
-      res <- res / sz
-    }
-    res
-  }
-  ressizes <- mapply(getEstimatedSize, allnames, allvarnum, allnullable, alltypes,
-                     MoreArgs=list(qryptr=qryptr), SIMPLIFY=TRUE)
-  resrv <- max(1, ressizes) # ensure >0 for correct handling of zero-length outputs
-  ## allocate and set buffers
-  getBuffer <- function(name, type, varnum, nullable, resrv, qryptr, arrptr) {
-      if (is.na(varnum)) {
-          if (type %in% c("CHAR", "ASCII", "UTF8")) {
-              buf <- libtiledb_query_buffer_var_char_alloc_direct(resrv, resrv*8, nullable)
-              qryptr <- libtiledb_query_set_buffer_var_char(qryptr, name, buf)
-              buf
+  buflist <- vector(mode="list", length=length(allnames))
+
+  if (length(x@buffers) != 0) {         # if we were given buffers (as in the case of TileDB Cloud ops)
+      nm <- names(x@buffers)
+      if (!isTRUE(all.equal(nm,allnames)))
+          stop("Expected ", paste(allnames, collapse=","), " got ", paste(nm, collapse=","), call. = FALSE)
+      for (i in seq_along(allnames)) {
+          n <- allnames[i]
+          path <- x@buffers[[n]]
+          if (!file.exists(path)) stop("No buffer for ", n, call. = FALSE)
+          if (is.na(allvarnum[i])) {
+              buflist[[i]] <- vlcbuf_from_shmem(path, alltypes[i])
           } else {
-              message("Non-char var.num columns are not currently supported.")
+              buflist[[i]] <- querybuf_from_shmem(path, alltypes[i])
           }
-      } else {
-          buf <- libtiledb_query_buffer_alloc_ptr(arrptr, type, resrv, nullable)
-          qryptr <- libtiledb_query_set_buffer_ptr(qryptr, name, buf)
-          buf
       }
-  }
-  buflist <- mapply(getBuffer, allnames, alltypes, allvarnum, allnullable,
-                    MoreArgs=list(resrv=resrv, qryptr=qryptr, arrptr=arrptr),
-                    SIMPLIFY=FALSE)
 
-  ## if we have a query condition, apply it
-  if (isTRUE(x@query_condition@init)) {
-      qryptr <- libtiledb_query_set_condition(qryptr, x@query_condition@ptr)
-  }
+      ## get results (shmem variant)
+      getResultShmem <- function(buf, name, varnum) { #, resrv, qryptr) {
+          if (is.na(varnum)) {
+              ##vec <- libtiledb_query_result_buffer_elements_vec(qryptr, name)
+              vec <- length_from_vlcbuf(buf)
+              libtiledb_query_get_buffer_var_char(buf, vec[1], vec[2])[,1]
+          } else {
+              libtiledb_query_get_buffer_ptr(buf, asint64)
+          }
+      }
+      reslist <- mapply(getResultShmem, buflist, allnames, allvarnum, SIMPLIFY=FALSE)
+      ## convert list into data.frame (cheaply)
+      res <- data.frame(reslist)
+      colnames(res) <- allnames
 
-  ## fire off query
-  qryptr <- libtiledb_query_submit(qryptr)
+  } else {                         # -- start 'big else' of standard query build
 
-  ## check status
-  status <- libtiledb_query_status(qryptr)
-  if (status != "COMPLETE") warning("Query returned '", status, "'.", call. = FALSE)
+      ## retrieve est_result_size
+      getEstimatedSize <- function(name, varnum, nullable, qryptr, datatype) {
+          if (is.na(varnum) && !nullable)
+              res <- libtiledb_query_get_est_result_size_var(qryptr, name)[1]
+          else if (is.na(varnum) && nullable)
+              res <- libtiledb_query_get_est_result_size_var_nullable(qryptr, name)[1]
+          else if (!is.na(varnum) && !nullable)
+              res <- libtiledb_query_get_est_result_size(qryptr, name)
+          else if (!is.na(varnum) && nullable)
+              res <- libtiledb_query_get_est_result_size_nullable(qryptr, name)[1]
+          if (rangeunset && tiledb::tiledb_version(TRUE) >= "2.2.0") {
+              sz <- tiledb_datatype_string_to_sizeof(datatype)
+              res <- res / sz
+          }
+          res
+      }
+      ressizes <- mapply(getEstimatedSize, allnames, allvarnum, allnullable, alltypes,
+                         MoreArgs=list(qryptr=qryptr), SIMPLIFY=TRUE)
+      resrv <- max(1, ressizes) # ensure >0 for correct handling of zero-length outputs
+      ## allocate and set buffers
+      getBuffer <- function(name, type, varnum, nullable, resrv, qryptr, arrptr) {
+          if (is.na(varnum)) {
+              if (type %in% c("CHAR", "ASCII", "UTF8")) {
+                  buf <- libtiledb_query_buffer_var_char_alloc_direct(resrv, resrv*8, nullable)
+                  qryptr <- libtiledb_query_set_buffer_var_char(qryptr, name, buf)
+                  buf
+              } else {
+                  message("Non-char var.num columns are not currently supported.")
+              }
+          } else {
+              buf <- libtiledb_query_buffer_alloc_ptr(arrptr, type, resrv, nullable)
+              qryptr <- libtiledb_query_set_buffer_ptr(qryptr, name, buf)
+              buf
+          }
+      }
+      buflist <- mapply(getBuffer, allnames, alltypes, allvarnum, allnullable,
+                        MoreArgs=list(resrv=resrv, qryptr=qryptr, arrptr=arrptr),
+                        SIMPLIFY=FALSE)
 
-  ## finalize query
-  qryptr <- libtiledb_query_finalize(qryptr)
+      ## if we have a query condition, apply it
+      if (isTRUE(x@query_condition@init)) {
+          qryptr <- libtiledb_query_set_condition(qryptr, x@query_condition@ptr)
+      }
 
-  ## close array
-  libtiledb_array_close(arrptr)
+      ## fire off query
+      qryptr <- libtiledb_query_submit(qryptr)
 
-  ## retrieve actual result size (from fixed size element columns)
-  getResultSize <- function(name, varnum, qryptr) {
-    if (is.na(varnum))                  # symbols come up with higher count
-      libtiledb_query_result_buffer_elements(qryptr, name, 0)
-    else
-      libtiledb_query_result_buffer_elements(qryptr, name)
-  }
-  estsz <- mapply(getResultSize, allnames, allvarnum, MoreArgs=list(qryptr=qryptr), SIMPLIFY=TRUE)
-  if (any(!is.na(estsz))) {
-      resrv <- max(estsz, na.rm=TRUE)
-  } else {
-      resrv <- resrv/8                  # character case where bytesize of offset vector was used
-  }
+      ## check status
+      status <- libtiledb_query_status(qryptr)
+      if (status != "COMPLETE") warning("Query returned '", status, "'.", call. = FALSE)
 
-  ## get results
-  getResult <- function(buf, name, varnum, resrv, qryptr) {
-    if (is.na(varnum)) {
-      vec <- libtiledb_query_result_buffer_elements_vec(qryptr, name)
-      libtiledb_query_get_buffer_var_char(buf, vec[1], vec[2])[,1]
-    } else {
-      libtiledb_query_get_buffer_ptr(buf, asint64)
-    }
-  }
-  reslist <- mapply(getResult, buflist, allnames, allvarnum,
-                    MoreArgs=list(resrv=resrv, qryptr=qryptr), SIMPLIFY=FALSE)
-  ## convert list into data.frame (cheaply) and subset
-  res <- data.frame(reslist)[seq_len(resrv),]
-  colnames(res) <- allnames
+      ## close array
+      libtiledb_array_close(arrptr)
+
+      ## retrieve actual result size (from fixed size element columns)
+      getResultSize <- function(name, varnum, qryptr) {
+          if (is.na(varnum))                  # symbols come up with higher count
+              libtiledb_query_result_buffer_elements(qryptr, name, 0)
+          else
+              libtiledb_query_result_buffer_elements(qryptr, name)
+      }
+      estsz <- mapply(getResultSize, allnames, allvarnum, MoreArgs=list(qryptr=qryptr), SIMPLIFY=TRUE)
+      if (any(!is.na(estsz))) {
+          resrv <- max(estsz, na.rm=TRUE)
+      } else {
+          resrv <- resrv/8                  # character case where bytesize of offset vector was used
+      }
+
+      ## get results
+      getResult <- function(buf, name, varnum, resrv, qryptr) {
+          has_dumpbuffers <- length(x@dumpbuffers) > 0
+          if (is.na(varnum)) {
+              vec <- libtiledb_query_result_buffer_elements_vec(qryptr, name)
+              if (has_dumpbuffers) {
+                  #cat("Name: ", name, " (", paste0(vec, collapse=","), ") ", sep="")
+                  vlcbuf_to_shmem(x@dumpbuffers, name, buf, vec)
+              }
+              ##print(vec)
+              libtiledb_query_get_buffer_var_char(buf, vec[1], vec[2])[,1]
+          } else {
+              if (has_dumpbuffers) {
+                  #cat("Name: ", name, " ", asint64, " ", resrv, " ", sep="")
+                  vecbuf_to_shmem(x@dumpbuffers, name, buf, resrv)
+              }
+              libtiledb_query_get_buffer_ptr(buf, asint64)
+          }
+      }
+      reslist <- mapply(getResult, buflist, allnames, allvarnum,
+                        MoreArgs=list(resrv=resrv, qryptr=qryptr), SIMPLIFY=FALSE)
+      ## convert list into data.frame (cheaply) and subset
+      res <- data.frame(reslist)[seq_len(resrv),,drop=FALSE]
+      colnames(res) <- allnames
+  }                                     # end of 'big else' for query build, submission and read
 
   ## reduce output if extended is false, or attrs given
   if (!x@extended) {
       if (length(sel) > 0) {
-          res <- res[, if (sparse) allnames else attrnames]
+          res <- res[, if (sparse) allnames else attrnames, drop=FALSE]
       }
       k <- match("__tiledb_rows", colnames(res))
       if (is.finite(k)) {
-          res <- res[, -k]
+          res <- res[, -k, drop=FALSE]
       }
   }
 
@@ -962,7 +1000,7 @@ setMethod("[<-", "tiledb_array",
   nc <- if (is.list(value)) length(value) else ncol(value)
   nm <- if (is.list(value)) names(value) else colnames(value)
 
-  if (all.equal(sort(allnames),sort(nm))) {
+  if (isTRUE(all.equal(sort(allnames),sort(nm)))) {
 
     if (libtiledb_array_is_open_for_writing(x@ptr)) { 			# if open for writing
       arrptr <- x@ptr                                           #   use array
